(*====================================================================
    File rDialog.pas

    Components library for Delphi and BCB++.
      .
    Dialog windows interface.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rDialog.pas,v 1.3 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)

unit rDialog;

interface

uses Windows, Messages;

type
  TFontPitch = (fpDefault, fpVariable, fpFixed);
  TFontName = type string;
  TFontCharset = 0..255;
  TFontDataName = string[LF_FACESIZE - 1];
  TFontStyle = (fsBold, fsItalic, fsUnderline, fsStrikeOut);
  TFontStylesBase = set of TFontStyle;

  TFontData = record
    Handle: HFont;
    Height: Integer;
    Pitch: TFontPitch;
    Style: TFontStylesBase;
    Charset: TFontCharset;
    Name: TFontDataName;
  end;

  TFontItem = record
    ItemID:integer;
    FFontItem:HFont;
  end;

  TFontArray = array of TFontItem;
  TrDialogNotifyEvent = procedure (aSender: TObject) of object;
  TrDialogControlEvent = procedure (aSender: TObject; const aSenderID, aSenderParam: Integer) of object;
  TrDialogMessageEvent = function (aSender: TObject; const aMsg: WORD; const wParam, lParam: Integer): BOOL of object;
  TrCloseQueryEvent = procedure (aSender: TObject; var CanClose: Boolean) of object;

  TrCloseAction = (caNone, caHide, caFree, caMinimize);
  TrCloseEvent = procedure (aSender: TObject; var Action: TrCloseAction) of object;

  TrDialog = class
  private
    FFontCount: Integer;
    FFontArray: TFontArray;
    FDialogResource: string;
    FModalResult: Integer;
    FOwnerWindow: HWND;
    FHandle: HWND;
    FWindowProc: Pointer;
    FOnShow: TrDialogNotifyEvent;
    FOnControlEvent: TrDialogControlEvent;
    FOnKeyDown: TrDialogControlEvent;
    FOnKeyUp: TrDialogControlEvent;
    FOnClose: TrCloseEvent;
    FOnMessage: TrDialogMessageEvent;
    FOnCloseQuery: TrCloseQueryEvent;
    FErrorMessage: string;
    FIsModal,
    FShowError: Boolean;

    procedure _SetError;
    procedure InitWindowProc();
    function WindowProc(hWndDlg: HWND; uMsg: WORD; wParam: WPARAM; lParam: LPARAM): BOOL; stdcall;
    function GetItemHandle(const aItemID: Integer): HWND;
    function GetItemCaption(const aItemID: Integer): string;
    procedure SetItemCaption(const aItemID: Integer; const aCaption: string);
    procedure SetCaption(const aCaption: string);
    function GetItemInt(const aItemID: Integer): Integer;
    procedure SetItemInt(const aItemID, aInt: Integer);
    procedure CreateNewFont(const aItemID: Integer; CDataFont: TFontData);
    procedure FreeFonts();
    procedure DestroyHandle();
    function GetItemEnabled(aItemID: Integer): Boolean;
    procedure SetItemEnabled(aItemID: Integer; const aValue: Boolean);
    function GetEnabled(): Boolean;
    procedure SetEnabled(const aValue: Boolean);
  public
    constructor Create(const aDialogResource:string);
    function ShowModal(): Integer;
    procedure Show();
    procedure Hide();
    procedure Close();
    procedure SetFontItem(const aItemID:Integer;CDataFont:TFontData);
    function GetFontItem(const aItemID:integer):HFONT;
    destructor Destroy(); override;
    property OwnerWindow: HWND read FOwnerWindow write FOwnerWindow;
    property Handle: HWND read FHandle;
    property Caption: string write SetCaption;
    property ItemHandle[const aItemID: Integer]: HWND read GetItemHandle;
    property ItemCaption[const aItemID: Integer]: string read GetItemCaption write SetItemCaption;
    property ItemInt[const aItemID: Integer]: Integer read GetItemInt write SetItemInt;
    property ItemEnabled[aItemID: Integer]: Boolean read GetItemEnabled write SetItemEnabled;
    property ModalResult: Integer read FModalResult;
    property OnShow: TrDialogNotifyEvent read FOnShow write FOnShow;
    property OnMessage: TrDialogMessageEvent read FOnMessage write FOnMessage;
    property OnControlEvent: TrDialogControlEvent read FOnControlEvent write FOnControlEvent;
    property OnKeyDown: TrDialogControlEvent read FOnKeyDown write FOnKeyDown;
    property OnKeyUp: TrDialogControlEvent read FOnKeyUp write FOnKeyUp;
    property OnClose: TrCloseEvent read FOnClose write FOnClose;
    property OnCloseQuery: TrCloseQueryEvent read FOnCloseQuery write FOnCloseQuery;
    property ErrorMessage: string read FErrorMessage;
    property ShowError: Boolean read FShowError write FShowError;
    property Enabled: Boolean read GetEnabled write SetEnabled;
  end;

implementation

constructor TrDialog.Create(const aDialogResource: string);
begin
  FFontCount := 0;
  FIsModal := True;
  FDialogResource := aDialogResource;
  InitWindowProc;
end;

procedure TrDialog.InitWindowProc;
asm
  push esi
  push edi

  push eax        //Self
  lea  ecx, @@StartProc
  lea  eax, @@EndProc
  sub  eax, ecx
  push eax        //size
  call SysGetMem
  mov  edi, eax   //address
  pop  ecx        //size
  pop  eax        //Self
  push edi        //address
  mov  [eax + FWindowProc], edi
  lea  esi, @@StartProc
  cld
  rep  movsb
  pop  edx        //address
  inc  edx
  mov  [edx], eax //Self

  pop  edi
  pop  esi
  ret

  @@StartProc:
  mov  eax, $12345678  //Self stub
  pop  ecx             //return address
  push eax
  push ecx
  lea  eax, TrDialog.WindowProc
  jmp  eax
  @@EndProc:
end;

procedure TrDialog.FreeFonts;
var
  i: Integer;
begin
  for i := 0 to FFontCount - 1 do begin
    DeleteObject(FFontArray[i].FFontItem);
  end;
  SetLength(FFontArray, 0);
end;

function TrDialog.GetFontItem(const aItemID:integer):HFONT;
var
  i: Integer;
begin
  Result := 0;
  for i := 0 to length(FFontArray) - 1 do
  if (FFontArray[i].ItemID = aItemID) then
     Result := FFontArray[i].FFontItem;
end;

procedure TrDialog.CreateNewFont(const aItemID: Integer; CDataFont: TFontData);
var
  bfsItalic, bfsUnderline, bfsStrikeOut, num, vFontWeight: Integer;
begin
  //       ,  handle
  bfsItalic := 0;
  bfsUnderline := 0;
  bfsStrikeOut := 0;
  if ((fsBold) in CDataFont.Style)
    then vFontWeight := FW_BOLD
    else vFontWeight := FW_NORMAL;
  if (fsItalic) in CDataFont.Style
    then bfsItalic := 1;
  if (fsUnderline) in CDataFont.Style
    then bfsUnderline := 1;
  if (fsStrikeOut) in CDataFont.Style
    then bfsStrikeOut := 1;
  SetLength(FFontArray, length(FFontArray) + 1);
  num := Length(FFontArray) - 1;
  with FFontArray[num] do begin
    FFontItem := CreateFont(CDataFont.Height, 0, 0, 0, vFontWeight, bfsItalic,
      bfsUnderLine, bfsStrikeOut,CDataFont.Charset, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
      DEFAULT_QUALITY, DEFAULT_PITCH, @CDataFont.Name);
    ItemID := aItemID;
  end;
  Inc(FFontCount);
end;

procedure TrDialog.SetFontItem(const aItemID:Integer; CDataFont: TFontData);
begin
  CreateNewFont(aItemID, CDataFont);
  SendMessage(GetItemHandle(aItemID), WM_SETFONT, FFontArray[FFontCount - 1].FFontItem, MAKELONG(1, 0));
end;

procedure TrDialog._SetError;
var
  en: DWORD;
  pBuf: PChar;
  s: string;
begin
  en := GetLastError;
  if FormatMessage(
    FORMAT_MESSAGE_ALLOCATE_BUFFER or FORMAT_MESSAGE_FROM_SYSTEM or FORMAT_MESSAGE_IGNORE_INSERTS,
    nil, en, 0, @pBuf, 0, nil) = 0
  then begin
    str(en, s);
    FErrorMessage := ' ' + s;
  end
  else begin
    FErrorMessage := pBuf;
    LocalFree(DWORD(pBuf));
  end;
  if FShowError then MessageBox(0, PChar(ErrorMessage), '', MB_OK + MB_ICONERROR);
end;

function TrDialog.WindowProc(hWndDlg: HWND; uMsg: WORD; wParam: WPARAM; lParam: LPARAM): BOOL;
begin
  if Assigned(FOnMessage) then begin
    Result := FOnMessage(Self, uMsg, wParam, lParam);
    if Result then Exit;
  end;
  Result := False;
  case uMsg of
    WM_INITDIALOG: begin
      FHandle := hWndDlg;
      if Assigned(FOnShow) then FOnShow(Self);
    end;
    WM_COMMAND: begin
      if Assigned(FOnControlEvent) then FOnControlEvent(Self, wParam, lParam);
    end;
    WM_KEYDOWN: if Assigned(FOnKeyDown) then FOnKeyDown(Self, wParam, lParam);
    WM_KEYUP: if Assigned(FOnKeyUp) then FOnKeyUp(Self, wParam, lParam);
    WM_CLOSE: begin
      Close();
      Result := True;
    end;
  end;
end;

function TrDialog.ShowModal(): Integer;
begin
  FHandle := 0;
  FErrorMessage := '';
  FIsModal := True;
  FModalResult := DialogBoxParam(
    hInstance, MAKEINTRESOURCE(FDialogResource), FOwnerWindow, FWindowProc, 0);
  FHandle := 0;
  if FModalResult = -1 then _SetError;
  Result := FModalResult;
end;

procedure TrDialog.Show();
begin
  if FHandle = 0
    then FHandle := CreateDialog(hInstance, MAKEINTRESOURCE(FDialogResource), FOwnerWindow, FWindowProc);
  ShowWindow(FHandle, SW_SHOWNORMAL);
  UpdateWindow(FHandle);
  FIsModal := False;
end;

procedure TrDialog.Hide();
begin
  ShowWindow(FHandle, SW_HIDE);
end;

procedure TrDialog.DestroyHandle();
begin
  if FIsModal
    then EndDialog(Handle, 0)
    else DestroyWindow(FHandle);
  FHandle := 0;
end;

procedure TrDialog.Close();
var
  CanClose: Boolean;
  vAction: TrCloseAction;
begin
  if BOOL(Handle) then begin
    CanClose := True;
    if Assigned(FOnCloseQuery)
      then FOnCloseQuery(Self, CanClose);
    if CanClose then begin
      vAction := caNone;
      if Assigned(FOnClose) then FOnClose(Self, vAction);
      case vAction of
        caNone: begin
          if FIsModal then DestroyHandle()
          else Hide();
        end;
        caHide: Hide();
        caFree: begin
          DestroyHandle();
          Free();
        end;
        caMinimize: ShowWindow(FHandle, SW_MINIMIZE);
      end;
    end;
  end;
end;

procedure TrDialog.SetCaption(const aCaption: string);
begin
  if BOOL(Handle) then
    SendMessage(Handle, WM_SETTEXT, 0, DWORD(PChar(aCaption)));
end;

function TrDialog.GetItemHandle(const aItemID: Integer): HWND;
begin
  Result := GetDlgItem(Handle, aItemID);
end;

function TrDialog.GetItemCaption(const aItemID: Integer): string;
var
  p: PChar;
begin
  Result := '';
  if BOOL(Handle) then begin
    GetMem(p, 65535);
    try
      if (GetDlgItemText(Handle, aItemID, p, 65534) = 0) and (GetLastError() <> 0)
        then _SetError
        else Result := p;
    finally
      FreeMem(p);
    end;
  end;
end;

procedure TrDialog.SetItemCaption(const aItemID: Integer; const aCaption: string);
begin
  if BOOL(Handle) and not SetDlgItemText(Handle, aItemID, PChar(aCaption))
    then _SetError;
end;

function TrDialog.GetItemInt(const aItemID: Integer): Integer;
var
  r: BOOL;
begin
  Result := GetDlgItemInt(Handle, aItemID, r, True);
  if not r then _SetError;
end;

procedure TrDialog.SetItemInt(const aItemID, aInt: Integer);
begin
  if not SetDlgItemInt(Handle, aItemID, aInt, True)
    then _SetError;
end;

function TrDialog.GetItemEnabled(aItemID: Integer): Boolean;
begin
  Result := IsWindowEnabled(GetItemHandle(aItemID));
end;

procedure TrDialog.SetItemEnabled(aItemID: Integer; const aValue: Boolean);
begin
  EnableWindow(GetItemHandle(aItemID), aValue);
end;

function TrDialog.GetEnabled: Boolean;
begin
  Result := IsWindowEnabled(Handle);
end;

procedure TrDialog.SetEnabled(const aValue: Boolean);
begin
  EnableWindow(Handle, aValue);
end;

destructor TrDialog.Destroy;
begin
  if BOOL(FHandle)
  then DestroyHandle;
  FreeFonts;
  FreeMem(FWindowProc);
  inherited;
end;

end.


