(*====================================================================
    File rgraph.pas

    Components library for Delphi and BCB++.
    The TImage descendant.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rGraph.pas,v 1.3 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)

unit rGraph;

{$I POLARIS.INC}

interface

uses Messages, Windows, SysUtils, Classes, Controls, Graphics,
     {$IFDEF POLARIS_D4}ImgList,{$ENDIF}
     rUtils, rVCLUtils, ExtCtrls;

type

  TrImage = class(TImage)
  private
    FImageChangeLink: TChangeLink;
    FInternalImages: TCustomImageList;
    FImages: TCustomImageList;
    FImageIndex: Integer;
    FResKind: TBitmapResKind;
    FChangeItems: Boolean;
    FResource: string;
    FSavePicture: TPicture;
    FTimerIndex: Integer;
    FInterval: Cardinal;
    LoadFromDef,
    FSlideShow: Boolean;
    FTimer: TTimer;
    procedure ImageListChange(Sender: TObject);
    procedure SetImages(Value: TCustomImageList);
    procedure SetImageIndex(Value: Integer);
    procedure SetResKind(Value: TBitmapResKind);
    procedure SetResource(Value: String);
    function GetPicture: TPicture;
    procedure SetPicture(Value: TPicture);
    procedure SetSavePicture(Value: TPicture);
    procedure SetInterval(Value: Cardinal);
    procedure SetSlideShow(Value: Boolean);
    function GetCount: integer;
    procedure TimerExpired(Sender: TObject);
    procedure SavePictureChange(Sender: TObject);
  protected
    procedure CheckImageList(Index: Integer); virtual;
    procedure GetInternalImagelist; virtual;
    function CurrentImages: TCustomImageList; virtual;
    procedure CheckTimer; virtual;
    procedure LoadAndResizeImage(Image: TCustomImageList; RHandle: THandle;
                                 ResName: string; ResKind: TBitmapResKind;
                                 Transparent: Boolean);
    procedure Notification(AComponent: TComponent;
                           Operation: TOperation); override;


  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Count: Integer read GetCount;
  published
    property DefPicture: TPicture read FSavePicture write SetSavePicture;
    property Images: TCustomImageList read FImages write SetImages;
    property ImageIndex: Integer read FImageIndex write SetImageIndex default -1;
    property ResKind: TBitmapResKind read FResKind write SetResKind default DefBRK;
    property Resource: String read FResource write SetResource;
    property Picture: TPicture read GetPicture write SetPicture stored False;
    property Interval: Cardinal read FInterval write SetInterval default 250;
    property SlideShow: Boolean read FSlideShow write SetSlideShow default False;
  end;

implementation

type
  TCrackImageList = class(TCustomImageList);

{ TrImage }

procedure TrImage.LoadAndResizeImage(Image: TCustomImageList; RHandle: THandle;
                                     ResName: string; ResKind: TBitmapResKind;
                                     Transparent: Boolean);
var
  TB: TBitmap;
  TRC: TColor;
  S: Integer;
begin
  if Assigned(Image) then begin
    TB := TBitmap.Create;
    try
      try
        LoadBitmapFromResource(TB, hInstance, ResName, rkBitmap);
      except
      on EResNotFound do if not(csDesigning in ComponentState) then raise;
      end;
      S := TB.Width;
      if S > TB.Height then S := TB.Height;
      TCrackImageList(Image).Width  := S;
      TCrackImageList(Image).Height := S;
      TRC := clNone;
//      if Transparent then TRC := TB.TransparentColor;
      Image.Clear;
      Image.AddMasked(TB, TRC);
    finally
      TB.Free;
    end;
  end;
end;

constructor TrImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FResKind := DefBRK;
  LoadFromDef := True;
  FChangeItems:= False;
  FSlideShow := False;
  FTimerIndex := -1;
  FInterval := 250;
  FSavePicture:= nil;
  FImageIndex := -1;
  FImages := nil;
  FTimer := nil;
  FSavePicture := TPicture.Create;
  FSavePicture.OnChange := SavePictureChange;
  FImageChangeLink := TChangeLink.Create;
  FImageChangeLink.OnChange := ImageListChange;
  ControlStyle := ControlStyle + [csReplicatable];
  FInternalImages :=  TCustomImageList.Create(Self);
  GetInternalImageList;
  FInternalImages.RegisterChanges(FImageChangeLink);
  FInternalImages.FreeNotification(Self);
end;

destructor TrImage.Destroy;
begin
  if Assigned(Images) then Images.UnRegisterChanges(FImageChangeLink);
  if Assigned(FTimer) then FTimer.Free;
  FInternalImages.UnRegisterChanges(FImageChangeLink);
  FInternalImages.Free;
  FImageChangeLink.Free;
  if Assigned(FSavePicture) then FSavePicture.Free;
  inherited Destroy;
end;

procedure TrImage.CheckImageList(Index: Integer);
var
  CI: TCustomImageList;
  Bmp: TBitmap;
begin
  FChangeItems := True;
  try
    CI := CurrentImages;
    if (Index < 0) or (Index > Pred(Count))
    then Picture.Assign(FSavePicture)
    else begin
      Bmp := TBitmap.Create;
      try
        CI.GetBitmap(Index, Bmp);
        inherited Picture.Assign(Bmp);
      finally
        Bmp.Free;
      end;
    end;
    Invalidate;
  finally
    FChangeItems := False;
  end;
end;

procedure TrImage.GetInternalImagelist;
begin
  if Assigned(FImages) then Exit
  else begin
    if Assigned(FInternalImages) then begin
      FInternalImages.Clear;
      if (FResource <> EmptyStr) and (FResKind <> rkNone)
      then LoadAndResizeImage(FInternalImages, hInstance, PChar(FResource), FResKind, Transparent);
    end;
  end;
end;

procedure TrImage.ImageListChange(Sender: TObject);
begin
  if Sender is TCustomImageList then CheckImageList(FImageIndex);
end;

procedure TrImage.SetImageIndex(Value: Integer);
begin
  if FImageIndex <> Value then begin
    FImageIndex := Value;
    CheckImageList(FImageIndex);
  end;
end;

procedure TrImage.SetImages(Value: TCustomImageList);
begin
  if FImages <> Value then begin
    if Images <> nil then Images.UnRegisterChanges(FImageChangeLink);
    FImages := Value;
    if Images <> nil then begin
      Images.RegisterChanges(FImageChangeLink);
      Images.FreeNotification(Self);
    end;
    GetInternalImagelist;
    CheckImageList(FImageIndex);
  end;
end;

procedure TrImage.SetResKind(Value: TBitmapResKind);
begin
  if FResKind <> Value then begin
    FResKind := Value;
    GetInternalImagelist;
    CheckImageList(FImageIndex);
  end;
end;

procedure TrImage.SetResource(Value: String);
begin
  if FResource <> Value then begin
    FResource := Value;
    GetInternalImagelist;
    CheckImageList(FImageIndex);
  end;
end;

function TrImage.CurrentImages: TCustomImageList;
begin
  Result := FInternalImages;
  if Assigned(FImages) then Result := FImages;
end;

function TrImage.GetPicture: TPicture;
begin
  Result := inherited Picture;
end;

procedure TrImage.SetPicture(Value: TPicture);
begin
  if (FImageIndex < 0) or (FImageIndex > Pred(Count))
  then FSavePicture.Assign(Value);
end;

procedure TrImage.SetInterval(Value: Cardinal);
begin
  if FInterval <> Value then begin
    FInterval := Value;
    CheckTimer;
  end;
end;

procedure TrImage.CheckTimer;
begin
  if not Assigned(FTimer) then begin
    FTimer := TTimer.Create(Self);
    FTimer.OnTimer  := TimerExpired;
  end;
  FTimerIndex     := FImageIndex;
  FTimer.Interval := FInterval;
  FTimer.Enabled  := FSlideShow;
  if not FSlideShow then CheckImageList(FImageIndex);
end;

procedure TrImage.TimerExpired(Sender: TObject);
var
  Cnt: Integer;
begin
  if  Assigned(FTimer) and FTimer.Enabled then begin
    Cnt := Count;
    if (Cnt > 0) then begin
      FTimerIndex := (FTimerIndex+1) mod Cnt;
      if (FInterval > 0)
      then CheckImageList(FTimerIndex)
      else CheckImageList(FImageIndex);
    end;
  end;
end;

procedure TrImage.SetSlideShow(Value: Boolean);
begin
  if FSlideShow <> Value then begin
    FSlideShow := Value;
    CheckTimer;
  end;
end;

procedure TrImage.SetSavePicture(Value: TPicture);
begin
  FSavePicture.Assign(Value);
end;

function TrImage.GetCount: integer;
var
  Temp: TCustomImageList;
begin
  Result := 0;
  Temp := CurrentImages;
  if Assigned(Temp) then Result := Temp.Count;
end;

procedure TrImage.SavePictureChange(Sender: TObject);
begin
  CheckImageList(FImageIndex);
end;

procedure TrImage.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = FImages)
  then Images := nil;
end;

end.
